"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const request_promise_native_1 = __importDefault(require("request-promise-native"));
const framework_1 = require("../homebridge/framework");
const platform_config_1 = require("../platform_config");
var AuthState;
(function (AuthState) {
    AuthState["NoToken"] = "no_token";
    AuthState["HasToken"] = "has_token";
    AuthState["TokenExpired"] = "token_expired";
})(AuthState || (AuthState = {}));
class TionApi {
    constructor(log, config, authApi) {
        this.log = log;
        this.config = config;
        this.authApi = authApi;
        this.stateRequest = null;
    }
    init() {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.authApi.init();
        });
    }
    getSystemState() {
        return __awaiter(this, void 0, void 0, function* () {
            let firstRequest = false;
            if (!this.stateRequest) {
                this.log.debug('Loading system state');
                this.stateRequest = this._internalRequest('get', '/location', { json: true });
                firstRequest = true;
            }
            const ret = yield this.stateRequest;
            if (firstRequest) {
                this.stateRequest = null;
            }
            if (this.config.stationName) {
                const lower = this.config.stationName.toLowerCase();
                const location = ret.find(loc => loc.name.toLowerCase() === lower);
                if (location) {
                    return location;
                }
                else {
                    this.log.warn(`Location ${this.config.stationName} not found`);
                }
            }
            return ret[0];
        });
    }
    execCommand(deviceId, payload) {
        return __awaiter(this, void 0, void 0, function* () {
            this.log.debug(`TionApi.execCommand(deviceId = ${deviceId}, payload = ${JSON.stringify(payload)})`);
            try {
                let result = yield this._internalRequest('post', `/device/${deviceId}/mode`, { body: payload, json: true });
                const commandId = result.task_id;
                this.log.debug(`TionApi.execCommand(commandId = ${commandId}, result = ${JSON.stringify(result)})`);
                while (result.status !== 'completed') {
                    switch (result.status) {
                        default:
                            this.log.error(`Unknown command status`);
                            this.log.error(result);
                            throw new Error('Status');
                            break;
                        case 'delivered':
                        case 'queued':
                            // await new Promise((resolve) => setTimeout(resolve, 250));
                            result = yield this._internalRequest('get', `/task/${commandId}`, { json: true });
                            this.log.debug(`TionApi.execCommand(result = ${JSON.stringify(result)})`);
                            break;
                    }
                }
                return result;
            }
            catch (err) {
                this.log.error('Failed to execte command');
                this.log.error(err);
                throw err;
            }
        });
    }
    _internalRequest(method, endpoint, options = {}) {
        return __awaiter(this, void 0, void 0, function* () {
            let accessToken = this.authApi.getAccessToken();
            let state = accessToken ? AuthState.HasToken : AuthState.NoToken;
            while (true) {
                switch (state) {
                    case AuthState.HasToken:
                        try {
                            const headers = Object.assign({}, options.headers || {}, {
                                Authorization: `Bearer ${accessToken}`,
                            });
                            const result = yield request_promise_native_1.default[method](`${TionApi.ApiBasePath}${endpoint}`, Object.assign(Object.assign({}, options), { headers }));
                            return result;
                        }
                        catch (err) {
                            if (err.statusCode === 401) {
                                this.log.error('TionApi  token_expired:', err.message || err.statusCode);
                                state = AuthState.TokenExpired;
                            }
                            else {
                                throw err;
                            }
                        }
                        break;
                    case AuthState.NoToken:
                        try {
                            accessToken = yield this.authApi.authenticateUsingPassword();
                            state = AuthState.HasToken;
                        }
                        catch (err) {
                            this.log.error('TionApi - no_token:', err.message || err.statusCode);
                            throw err;
                        }
                        break;
                    case AuthState.TokenExpired:
                        try {
                            accessToken = yield this.authApi.authenticateUsingRefreshToken();
                            state = AuthState.HasToken;
                        }
                        catch (err) {
                            this.log.error('TionApi - token_expired:', err.message || err.statusCode);
                            if (err.statusCode >= 400 && err.statusCode < 500) {
                                state = AuthState.NoToken;
                            }
                            else {
                                throw err;
                            }
                        }
                        break;
                }
            }
        });
    }
}
exports.TionApi = TionApi;
TionApi.ApiBasePath = 'https://api2.magicair.tion.ru';
//# sourceMappingURL=api.js.map